package info.bukova.isspst.ui.main;

import info.bukova.isspst.Constants;
import info.bukova.isspst.SessionData;
import info.bukova.isspst.StringUtils;
import info.bukova.isspst.data.RequirementBase;
import info.bukova.isspst.reporting.Generator;
import info.bukova.isspst.reporting.GeneratorFactory;
import info.bukova.isspst.reporting.ParamFiller;
import info.bukova.isspst.reporting.Report;
import info.bukova.isspst.reporting.ReportDefinition;
import info.bukova.isspst.services.Service;
import info.bukova.isspst.services.requirement.ApproveException;
import info.bukova.isspst.services.requirement.RequirementBaseService;
import info.bukova.isspst.signapi.SignData;
import info.bukova.isspst.ui.ListViewModel;
import org.zkoss.bind.BindUtils;
import org.zkoss.bind.annotation.BindingParam;
import org.zkoss.bind.annotation.Command;
import org.zkoss.bind.annotation.ExecutionArgParam;
import org.zkoss.bind.annotation.Init;
import org.zkoss.bind.annotation.NotifyChange;
import org.zkoss.zk.ui.Executions;
import org.zkoss.zk.ui.select.annotation.WireVariable;
import org.zkoss.zul.Messagebox;
import org.zkoss.zul.Window;

import java.util.Date;
import java.util.UUID;

/**
 * @author Pepa Rokos
 */
public class ApproveDialogVM {

	private RequirementBaseService service;
	private RequirementBase requirement;
	private ListViewModel grid;
	private Date approveDate;
	@WireVariable
	private SessionData sessionData;
	@WireVariable
	private ReportDefinition reportDefinition;
	@WireVariable
	private GeneratorFactory genFactory;
	@WireVariable
	private ParamFiller paramFiller;
	private boolean signed;
	private boolean timer;

	public Date getApproveDate() {
		return approveDate;
	}

	public void setApproveDate(Date approveDate) {
		this.approveDate = approveDate;
	}

	@Init
	public void init(@ExecutionArgParam("service") RequirementBaseService service,
					 @ExecutionArgParam("requirement") RequirementBase requirement,
					 @ExecutionArgParam("grid") ListViewModel grid) {
		this.service = service;
		this.requirement = requirement;
		this.grid = grid;
		this.approveDate = new Date();

		this.signed = false;
		this.timer = false;
	}

	@Command
	public void approve(@BindingParam("window") Window window) {
		try {
			service.approve(requirement, approveDate);
			BindUtils.postNotifyChange(null, null, grid, "dataBean");
			BindUtils.postNotifyChange(null, null, grid, "canApprove");
			BindUtils.postGlobalCommand(null, null, "reload", null);
			window.detach();
		} catch (ApproveException ex) {
			Messagebox.show(StringUtils.localize(ex.getReason()), StringUtils.localize("Error"), Messagebox.OK,
					Messagebox.ERROR);
		}
	}

	@Command
	@NotifyChange("timer")
	public void signPdf() {
		SignData data = new SignData();
		sessionData.setProperty(Constants.KEY_SIGN_DATA, data);
		data.setSignGuid(UUID.randomUUID().toString());
		sessionData.setProperty(Constants.KEY_SIGN_GUID, data.getSignGuid());

		// ToDo: Generovat report
		reportDefinition.clear();
		reportDefinition.setSingleObject(requirement);
		reportDefinition.setReport(new Report(0, true, "", "tripRequirementApp", false, true, true));
		reportDefinition.setService((Service) service);

		paramFiller.fill();
		Generator gen = genFactory.createGenerator(reportDefinition);
		data.setPdfData(gen.generate());
		data.setDescription(requirement.getDescription());
		data.setNumser(requirement.getNumser());

		timer = true;

		Executions.getCurrent().sendRedirect("/api/sign/signer.jnlp", "signer");
	}

	@Command
	@NotifyChange({"signed", "timer"})
	public void onTimer() {
		String guid = (String) sessionData.getProperty(Constants.KEY_SIGN_GUID);
		SignData data = (SignData) sessionData.getProperty(Constants.KEY_SIGN_DATA);

		if (guid.equals(data.getSignGuid()) && data.isSignSuccess()) {
			signed = true;
			timer = false;
		} else {
			signed = false;
		}
	}

	public boolean isSigned() {
		return signed;
	}

	public boolean isTimer() {
		return timer;
	}

}
